PROGRAM Heater

!   Simulates two-dimensional thermal histories,
!   using the Explicit or the Alternating-Direction-Implicit
!   variant of the Finite-Difference method.

!   Includes an option to advect material diagonally across the grid
!   to model subduction.

!   Written by Peter Bird, Department of Earth and Space Sciences,
!      University of California, Los Angeles.
!  (C) Copyright 1987, 1988, 1994, 2000 by
!      Peter Bird and the Regents of the
!      University of California.

      IMPLICIT NONE
INTERFACE
SUBROUTINE Confuse (conduc, dt, dx, dz, heatpr, kappa, &
     &              nrd, nx, nz, & ! intent(IN)
     &              tgrid0)        ! intent(IN OUT)
      IMPLICIT NONE
      REAL, INTENT(IN) :: conduc, dt, dx, dz, heatpr, kappa
      INTEGER, INTENT(IN) :: nrd, nx, nz
      REAL, DIMENSION(nrd,*), INTENT(IN OUT) :: tgrid0
END SUBROUTINE Confuse

SUBROUTINE Diffuse (conduc, dt, dx, dz, heatpr, kappa, &
     &              nrd, nx, nz, & ! intent(IN)
     &              tgrid0)        ! intent(IN OUT)
      IMPLICIT NONE
      REAL, INTENT(IN) :: conduc, dt, dx, dz, heatpr, kappa
      INTEGER, INTENT(IN) :: nrd, nx, nz
      REAL, DIMENSION(nrd,*), INTENT(IN OUT) :: tgrid0
END SUBROUTINE Diffuse

SUBROUTINE Advect (conduc, dt, dx, dz, kappa, &
     &             nrd, nx, nz, &
     &             stress, veloc, & ! intent(IN)
     &             tgrid0)  ! intent(IN OUT)
       IMPLICIT NONE
       REAL, INTENT(IN)    :: conduc, dt, dx, dz, kappa, stress, veloc
       INTEGER, INTENT(IN) :: nrd, nx, nz
       REAL, DIMENSION(nrd,*), INTENT(IN OUT) :: tgrid0
END SUBROUTINE Advect
END INTERFACE

      CHARACTER(79)             :: fout, fin, title
      INTEGER, PARAMETER        :: nrd = 99 ! maximum dimension of grids
      INTEGER                   :: i, iread, itime, j, nfound, npr, ntime, &
                                 & nx, nxmin, nz, option
      INTEGER, DIMENSION(nrd)   :: it
      LOGICAL                   :: intrud, restrt, savall, subduc
      REAL                      :: conduc, dipdeg, dt, dx, dz, heatpr, &
                                 & kappa, stress, time, veloc, &
                                 & xmax, xmin, &
                                 & z, zmax, zmin
      REAL, DIMENSION(nrd)      :: geoth0
      REAL, DIMENSION(nrd, nrd) :: tgrid

!   INPUT DATA

      WRITE (*, 1)
    1 FORMAT (' HEATER;'/ &
     &        ' Simulates 2-Dimensional (x,z) thermal histories'/ &
     &        '   by the Explicit variant, or'/ &
     &        '   by the Alternating-Direction-Implicit variant'/ &
     &        '   of the Finite-Difference method.'// &
     &        ' Boundary conditions are:'/ &
     &        '    -Fixed T on the top;'/ &
     &        '    -Fixed T on the bottom;'/ &
     &        '    -dT/dx = 0 along the sides.'// &
     &        ' Do you wish to model subduction in this run?'/ &
     &        '  (Enter T if you do, or F if you do not: ')
      READ (*, *) subduc
      intrud = subduc
      WRITE (*, 5)
    5 FORMAT (' Do you wish to read an old output (.grd) file for '/ &
     &        ' the initial temperatures (T/F)?: ')
      READ (*, *) restrt
      IF (restrt) THEN
           WRITE (*, 10)
   10      FORMAT (' Enter name of (.grd?) file containing array: ')
           READ 15, fin
   15      FORMAT (A)
           nfound = 0
           OPEN (UNIT = 10, FILE = fin)
           DO iread = 1, 999
                READ (10, *, END = 61) xmin, dx, xmax
                READ (10, *, END = 61) zmin, dz, zmax
                nfound = nfound + 1
                nx = (xmax - xmin) / dx + 1.5
                nz = (zmax - zmin) / dz + 1.5
                npr = MIN(nx, 15)
                READ (10, * ) ((tgrid(i, j), j = 1, nx), i = 1, nz)
                DO i = 1, nz
                     DO j = 1, npr
                          it(j) = tgrid(i, j) + 0.5
                     END DO
                     WRITE (*, 45) (it(j), j = 1, npr)
   45                FORMAT (' ',15I5)
                END DO
                READ (10, 15, END = 61) title
                WRITE (*, 55) TRIM(title)
   55           FORMAT (' ',A)
           END DO
   61      CLOSE (10)
           IF (nfound == 0) THEN
                WRITE (*, 70)
   70           FORMAT (' FILE EMPTY, OR INCORRECTLY FORMATTED.')
                STOP
           ELSE IF (nfound == 1) THEN
                WRITE (*, 71)
   71           FORMAT (' Initial temperature array was read.')
           ELSE
                WRITE (*, 72)
   72           FORMAT (' File has been read, down to the last array.')
           END IF
      ELSE
   74      WRITE (*, 75) nrd
   75      FORMAT (' Enter number of depths (including surface);' &
     &            /' (Number should be odd, and ', &
     &             'no greater than',I3,'): ')
           READ (*, *) nz
           IF ((nz < 3).OR.(nz > nrd).OR.(MOD(nz, 2) /= 1)) THEN 
                WRITE (*, "(' Error. Please follow instructions.')")
                GO TO 74
           END IF
           WRITE (*, 80)
   80      FORMAT (' Enter spacing of depth points: ')
           READ (*, *) dz
           IF (subduc) THEN
                nxmin = nz + 2
           ELSE
                nxmin = 3
           END IF
   84      WRITE (*, 85) nxmin, nrd
   85      FORMAT (' Enter number of columns of points;' &
     &            /'   (Number should be odd, at least ',I3, &
     &             ', and no more than ',I3,': ')
           READ (*, *) nx
           IF ((nx < nxmin).OR.(nx > nrd).OR.(MOD(nx, 2) /= 1)) THEN
                WRITE (*, "(' Error. Please follow instructions.')")
                GO TO 84
           END IF
           IF (subduc) THEN
   86           WRITE (*, 87)
   87           FORMAT (' Enter dip angle for subduction zone (5 to 85', &
     &                  ' degrees): ')
                READ (*, * ) dipdeg
                IF ((dipdeg < 5.).OR.(dipdeg > 85.)) THEN
                     WRITE (*, "(' Error. Please follow instructions.')")
                     GO TO 86
                END IF
                dx = dz / TAN(dipdeg / 57.296)
                WRITE (*, 88) dx
   88           FORMAT (' dx of ',ES12.4,' will be used.')
           ELSE
                WRITE (*, 90)
   90           FORMAT (' Enter lateral spacing of columns of points: ')
                READ (*, *) dx
           END IF
           xmin = 0.0
           xmax = dx * (nx - 1)
           zmin = dz * (1 - nz)
           zmax = 0.0
           WRITE (*, 95)
   95      FORMAT (' Enter initial temperature profile, top to bottom: ')
           DO i = 1, nz
                z = (i - 1) * dz
                WRITE (*, 100) i, z
  100           FORMAT (' Depth #',I5,' z =',ES12.4,' T =')
                READ (*, *) tgrid(i, 1)
           END DO
           DO j = 2, nx
                DO i = 1, nz
                     tgrid(i, j) = tgrid(i, 1)
                END DO
           END DO
      END IF
      WRITE (*, 145)
  145 FORMAT (' Enter time duration to be modelled: ')
      READ (*, *) time
      IF (time <= 0.0) THEN
           ntime = 0
      ELSE
          IF (subduc) THEN
               WRITE (*, 146)
  146          FORMAT (' Enter subduction velocity: ')
               READ (*, * ) veloc
               veloc = ABS(veloc)
               WRITE (*, 147)
  147          FORMAT (' Enter shear stress on subduction fault: ')
               READ (*, * ) stress
               stress = ABS(stress)
               WRITE (*, 148)
  148          FORMAT (' Do you wish to lock in the initial geothermal' &
     &                ,' gradient' &
     &                /' at the left side of the grid, as a crude way' &
     &                ,' of representing the' &
     &                /' heating effects of rising magma? (T/F): ')
               READ (*, * ) intrud
          END IF
  149     WRITE (*, 150)
  150     FORMAT (' Enter size of timestep: ')
          READ (*, *) dt
          IF (dt == 0.0) THEN
               WRITE (*, "(' Error. Timestep can NOT be zero.')")
               GO TO 149
          END IF
          ntime = time / dt + 0.5
          WRITE (*, 155) ntime
  155     FORMAT (' ',I10,' timesteps will be used.')
          WRITE (*, 160)
  160     FORMAT (' Enter diffusivity: ')
          READ (*, *) kappa
          WRITE (*, 165)
  165     FORMAT (' Enter conductivity: ')
          READ (*, *) conduc
          WRITE (*, 170)
  170     FORMAT (' Enter radioactive heat production (per unit volume): ')
          READ (*, *) heatpr
  175     WRITE (*, 176)
  176     FORMAT (' Enter solution method (0 for Explicit, 1 for ADI): ')
          READ (*, *) option
          IF ((option < 0).OR.(option > 1)) GO TO 175
          WRITE (*, 177)
  177     FORMAT (' Temperature array at end of computation is always', &
     &            ' printed to file;'/ &
     &            ' Do you want all intermediate timesteps recorded', &
     &            ' also? (T/F): ')
          READ (*, * ) savall
      END IF
      WRITE (*, 180)
  180 FORMAT (' Enter name of (.grd?) file to receive output T arrays: ')
      READ (*, 182) fout
  182 FORMAT (A)
      IF (intrud) THEN
           DO i = 1, nz
                geoth0(i) = tgrid(i, 1)
           END DO
      END IF

!     LOOP THAT ADVANCES TIME BY DT EACH PASS

      time = 0.0
      OPEN (UNIT = 11, FILE = fout)
      WRITE (*, *)
      IF (ntime == 0) THEN
           itime = 0
           WRITE (11, 186) xmin, dx, xmax, zmin, dz, zmax
  186      FORMAT (3ES12.4 &
     &            /3ES12.4)
           DO i = 1, nz
                DO j = 1, nx
                     it(j) = tgrid(i, j) + 0.5
                END DO
                WRITE  (11, 189) (it(j), j = 1, nx)
  189           FORMAT (100I5)
           END DO
           WRITE (11, 191) itime, time
  191      FORMAT ('^ At end of time step #',I5,' time =',ES13.5)
      END IF

      DO itime = 1, ntime
           time = time + dt
           IF (subduc) CALL Advect (conduc, dt, dx, dz, kappa, &
     &                              nrd, nx, nz, &
     &                              stress, veloc, & ! intent(IN)
     &                              tgrid)           ! intent(IN OUT)
           IF (option == 0) THEN

              CALL Confuse (conduc, dt, dx, dz, heatpr, kappa, &
     &                      nrd, nx, nz, & ! intent(IN)
     &                      tgrid)         ! intent(IN OUT)

           ELSE IF (option == 1) THEN

              CALL Diffuse (conduc, dt, dx, dz, heatpr, kappa, &
     &                      nrd, nx, nz, & ! intent(IN)
     &                      tgrid)         ! intent(IN OUT)

           END IF
           IF (intrud) THEN
                DO i = 1, nz - 1
                     tgrid(i, 1) = geoth0(i)
                END DO
           END IF
           IF (savall.OR.(itime == ntime)) THEN

!               OUTPUT (AT LEAST PART OF) T ARRAY TO SCREEN

                npr = MIN(nx, 15)
                DO i = 1, nz
                     DO j = 1, npr
                          it(j) = tgrid(i, j) + 0.5
                     END DO
                     WRITE (6, 290) (it(j), j = 1, npr)
  290                FORMAT (' ',15I5)
                END DO
                WRITE (6, 301) itime, time
  301           FORMAT (/'^ At end of time step #',I5,' time = ',ES13.5)

!               WRITE SECOND COPY OF OUTPUT TO A FILE

                WRITE (11, 310) xmin, dx, xmax, zmin, dz, zmax
  310           FORMAT (3ES12.4 &
     &                 /3ES12.4)
                DO i = 1, nz
                     DO j = 1, nx
                          it(j) = tgrid(i, j) + 0.5
                     END DO
                     WRITE (11, 391) (it(j), j = 1, nx)
  391                FORMAT (100I5)
                END DO
                WRITE (11, 410) itime, time
  410           FORMAT ('^ At end of time step #',I5,' time =',ES13.5)
           ELSE
                WRITE (*, 500) itime, ntime, time
  500           FORMAT ('+ Step ',I6,' out of ',I6,', time = ',ES13.5)
           END IF
      END DO

      CLOSE (11)
      STOP
END PROGRAM Heater



SUBROUTINE Confuse (conduc, dt, dx, dz, heatpr, kappa, &
     &              nrd, nx, nz, & ! intent(IN)
     &              tgrid0)        ! intent(IN OUT)

!   Update temperatures to reflect a time advance of dt,
!      replacing contents of tgrid0 and using tgrid1 as workspace.
!   Uses explicit variant of Finite-Difference method.
!   Dx and dz are grid spacings;
!   kappa is the diffusivity;
!   nrd is the first (row) dimension of tgrid0 and
!      tgrid1 in the calling program;
!   nx and nz are the numbers of points in each
!      row and column, respectively.

!   Important Note:
!      Buried in this code are the boundary conditions:
!         Unchanging temperature along top of grid,
!         Unchanging temperature along base of grid,
!         No heat conduction laterally through side walls.

      IMPLICIT NONE
      REAL, INTENT(IN) :: conduc, dt, dx, dz, heatpr, kappa
      INTEGER, INTENT(IN) :: nrd, nx, nz
      REAL, DIMENSION(nrd,*), INTENT(IN OUT) :: tgrid0

      INTEGER :: iz, jx
      REAL :: dc
      REAL, DIMENSION(:,:), ALLOCATABLE :: tgrid1
      ALLOCATE ( tgrid1(nz, nx) )

!     Density (d) times heat capacity (c) =
!     thermal-conductivity/diffusivity:
      IF (kappa > 0.0) THEN
           dc = conduc / kappa
      ELSE
           dc = 1.0
      END IF

!     COMPUTE NEW TEMPERATURES AND PLACE IN TGRID1

      DO iz = 2, nz-1
           DO jx = 1, nx
                IF (jx == 1) THEN
                     tgrid1(iz, jx) = tgrid0(iz, jx) + &
     &                            (heatpr / dc) * dt + &
     &                             kappa * dt * ( &
     &          (tgrid0(iz,   2)   - 2. * tgrid0(iz, 1) + tgrid0(iz,     2)) / dx**2 &
     &        + (tgrid0(iz - 1, 1) - 2. * tgrid0(iz, 1) + tgrid0(iz + 1, 1)) / dz**2 &
     &                                           )
!               ^ In statement above, note assumption T(iz,0)=T(iz,2).
!                 This places the mirror plane exactly on column 1.

                ELSE IF (jx == nx) THEN
                     tgrid1(iz, jx) = tgrid0(iz, jx) + &
     &                            (heatpr / dc) * dt + &
     &                             kappa * dt * ( &
     &          (tgrid0(iz, nx - 1) - 2. * tgrid0(iz, nx) + tgrid0(iz, nx - 1)) / dx**2 &
     &        + (tgrid0(iz - 1, nx) - 2. * tgrid0(iz, nx) + tgrid0(iz + 1, nx)) / dz**2 &
     &                                           )
!               ^ In statement above, note assumption that
!                    T(iz,nx+1)=T(iz,nx-1).
!                 This places the mirror plane exactly on column nx.

                ELSE
                     tgrid1(iz, jx) = tgrid0(iz, jx) + &
     &                            (heatpr / dc) * dt + &
     &                             kappa * dt * ( &
     &          (tgrid0(iz, jx - 1) - 2. * tgrid0(iz, jx) + tgrid0(iz, jx + 1)) / dx**2 &
     &        + (tgrid0(iz - 1, jx) - 2. * tgrid0(iz, jx) + tgrid0(iz + 1, jx)) / dz**2 &
     &                                           )
                END IF
           END DO
      END DO

!     MOVE COPY OF TEMPERATURES FROM WORKSPACE:TGRID1 TO OUTPUT:TGRID0

      DO iz = 2, nz-1
           DO jx = 1, nx
                tgrid0(iz, jx) = tgrid1(iz, jx)
           END DO
      END DO
      RETURN
END SUBROUTINE Confuse



SUBROUTINE Diffuse (conduc, dt, dx, dz, heatpr, kappa, &
     &              nrd, nx, nz, & ! intent(IN)
     &              tgrid0)        ! intent(IN OUT)

!   Update temperatures to reflect a time advance of dt,
!      replacing contents of tgrid0 and using tgrid1 as workspace.
!   Uses the Alternating-Direction-Implicit variatn of the F-D method.
!   Dx and dz are grid spacings;
!   kappa is the diffusivity;
!   nrd in the first (row) dimention of tgrid0 and
!      tgrid1 in the calling program;
!   nx and nz are the numbers of points in each
!      row and column, respectively.

!   Important Note:
!      Buried in this code are the boundary conditions:
!         Unchanging temperature along top of grid,
!         Unchanging temperature along base of grid,
!         No heat conduction laterally through side walls.

      IMPLICIT NONE

INTERFACE
SUBROUTINE TriSys (length, lowdia, maindi, topdia, & ! intent(IN)
     &             rights, & ! intent(IN OUT)
     &             answer)   ! intent(OUT)
      IMPLICIT NONE
      INTEGER, INTENT(IN) :: length
      REAL, DIMENSION(*), INTENT(IN)    :: lowdia, maindi, topdia
      REAL, DIMENSION(*), INTENT(IN OUT) :: rights
      REAL, DIMENSION(*), INTENT(OUT)   :: answer
END SUBROUTINE TriSys
END INTERFACE

      REAL, INTENT(IN) :: conduc, dt, dx, dz, heatpr, kappa
      INTEGER, INTENT(IN) :: nrd, nx, nz
      REAL, DIMENSION(nrd,*), INTENT(IN OUT) :: tgrid0

      INTEGER :: i, j
      REAL :: dxm2, dzm2, extra, factor, facx, facz, rhocp
      REAL, DIMENSION(:),   ALLOCATABLE :: answer, lowdia, maindi, &
                                         & rights, topdia
      REAL, DIMENSION(:,:), ALLOCATABLE :: tgrid1
      ALLOCATE ( answer(nrd), lowdia(nrd), maindi(nrd), &
              &  rights(nrd), topdia(nrd) )
      ALLOCATE ( tgrid1(nz, nx) )

!   PRECOMPUTE SOME COMMON FACTORS FOR GREATER SPEED:

      rhocp = conduc / kappa
      dxm2 = 1. / dx**2
      dzm2 = 1. / dz**2
      factor = 0.5 * dt * kappa
      facx = factor * dxm2
      facz = factor * dzm2
      extra = 0.5 * dt * heatpr / rhocp

!   ROW CALCULATIONS- IMPLICIT IN X, EXPLICIT IN Z

      DO j = 1, nx
           maindi(j) = 2. * facx + 1.
           topdia(j) = -facx
           lowdia(j) = -facx
      END DO
      topdia(1)  = -2. * facx
      lowdia(nx) = -2. * facx
      DO j = 1, nx
           tgrid1(1, j) = tgrid0(1, j)
           tgrid1(nz, j) = tgrid0(nz, j)
      END DO
      DO i = 2, nz-1
           DO j = 1, nx
                rights(j) = tgrid0(i, j) + facz * (   tgrid0(i-1, j) - &
     &                                           2. * tgrid0(i  , j) + &
     &                                                tgrid0(i+1, j)   ) + extra
           END DO
           CALL TriSys (nx, lowdia, maindi, topdia, & ! intent(IN)
     &                  rights, & ! intent(IN OUT)
     &                  answer)   ! intent(OUT)
           DO j = 1, nx
                tgrid1(i, j) = answer(j)
           END DO
      END DO

!   COLUMN CALCULATIONS- IMPLICIT IN Z, EXPLICIT IN X

      DO j = 1, nz
           maindi(j) = 2. * facz + 1.
           topdia(j) = -facz
           lowdia(j) = -facz
      END DO
      maindi(1)  = 1.
      topdia(1)  = 0.
      maindi(nz) = 1.
      lowdia(nz) = 0.
      DO j = 1, nx
           rights(1)  = tgrid1(1, j)
           rights(nz) = tgrid1(nz, j)
           IF (j == 1) THEN
                DO i = 2, nz-1
                     rights(i) = tgrid1(i, j) + facx * (   tgrid1(i, 2) - &
     &                                                2. * tgrid1(i, 1) + &
     &                                                     tgrid1(i, 2)   ) + extra
                END DO
           ELSE IF (j < nx) THEN
                DO i = 2, nz-1
                     rights(i) = tgrid1(i, j) + facx * (   tgrid1(i, j-1) - &
     &                                                2. * tgrid1(i, j  ) + &
     &                                                     tgrid1(i, j+1)   ) + extra
                END DO
           ELSE
                DO i = 2, nz-1
                    rights(i) = tgrid1(i, j) + facx * (   tgrid1(i, nx-1) - &
     &                                               2. * tgrid1(i, nx  ) + &
     &                                                    tgrid1(i, nx-1)   ) + extra
                END DO
           END IF
           CALL TriSys (nz, lowdia, maindi, topdia, & ! intent(IN)
     &                  rights, & ! intent(IN OUT)
     &                  answer)   ! intent(OUT)
           DO i = 1, nz
                tgrid0(i, j) = answer(i)
           END DO
      END DO
      DEALLOCATE ( answer, lowdia, maindi, &
                &  rights, topdia )
      RETURN
END SUBROUTINE Diffuse



SUBROUTINE TriSys (length, lowdia, maindi, topdia, & ! intent(IN)
     &             rights, & ! intent(IN OUT)
     &             answer)   ! intent(OUT)

!   Solves tri-diagonal systems of linear equations by Gauss elimination.
!   Length is the order of the system.

!   Matrix equation solved is: L(MT    )   (A)     (R)
!                               (OAO   )   (N)     (I)
!                               ( WIP  )   (S)  =  (G)
!                               (  DND )   (W)     (H)
!                               (   IDI)   (E)     (T)
!                               (    AI)A  (R)     (S)

!   So maindi is the main diagonal,
!      topdia is the upper co-diagonal,
!      lowdia is the lower co-diagonal,
!   and the subscript in each case refers to the row, not the column.
!   This mean that lowdia(1) and topdia(length) are not used.
!   Alos note that this routine preserves the left-hand matrix,
!      but clobbers the right-hand vector.

      IMPLICIT NONE
      INTEGER, INTENT(IN) :: length
      REAL, DIMENSION(*), INTENT(IN)    :: lowdia, maindi, topdia
      REAL, DIMENSION(*), INTENT(IN OUT) :: rights
      REAL, DIMENSION(*), INTENT(OUT)   :: answer

      INTEGER :: i
      REAL :: t
      REAL, DIMENSION(:), ALLOCATABLE :: dummy
      ALLOCATE ( dummy(length) )

      DO i = 1, length
           dummy(i) = maindi(i)
      END DO
      DO i = 2, length
           t = lowdia(i) / dummy(i - 1)
           dummy(i) = dummy(i) - topdia(i - 1) * t
           rights(i) = rights(i) - rights(i - 1) * t
      END DO
      answer(length) = rights(length) / dummy(length)
      DO i = length-1, 1, -1
           answer(i) = (rights(i) - topdia(i) * answer(i + 1)) / dummy(i)
      END DO
      RETURN
END SUBROUTINE TriSys



SUBROUTINE Advect (conduc, dt, dx, dz, kappa, &
     &             nrd, nx, nz, &
     &             stress, veloc, & ! intent(IN)
     &             tgrid0)   ! intent(IN OUT)

!   Transports temperatures in tgrid0 to represent subduction.
!   Uses tgrid1 for temporary storage.
!   The convention is that the subduction fault dips from upper right
!      to lower left, ending at the bottom left grid point.
!   Temperatures below the fault are translated by (veloc*dt),
!      horizontally and/or downdip.
!   Temperatures along the fault are the average of translated and
!      in-situ temperatures.
!   Heat is added to the diagonal row of nodes representing the
!      fault, in proportion to shear "stress".

       IMPLICIT NONE
       REAL, INTENT(IN)    :: conduc, dt, dx, dz, kappa, stress, veloc
       INTEGER, INTENT(IN) :: nrd, nx, nz
       REAL, DIMENSION(nrd,*), INTENT(IN OUT) :: tgrid0

       INTEGER :: irb, irow, irt, jcl, jcol, jcr, nfault
       REAL :: deltat, diag, dist, factor, fd, fr, &
     &         rhocp, sx, sz, t, tl, tr, &
     &         xcol, zrow
       REAL, DIMENSION(:,:), ALLOCATABLE :: tgrid1
       ALLOCATE ( tgrid1(nz, nx) )

       IF (kappa > 0.0) THEN
            rhocp = conduc / kappa
       ELSE
            rhocp = 1.0
       END IF
       dist = veloc * dt
       diag = SQRT(dx * dx + dz * dz)
       sx = dist / diag
       sz = dist / diag
       DO jcol = 1, nx
            nfault = MAX(nz - (jcol-1), 1)
            DO irow = 1, nz
                 IF (irow < nfault) THEN
                      tgrid1(irow, jcol) = tgrid0(irow, jcol)
                 ELSE
                      xcol = jcol + sx
                      xcol = MIN(xcol, (nx * 1.0))
                      zrow = irow - sz
                      zrow = MAX(zrow, 1.0 + (irow - nfault))
                      jcl = xcol
                      jcl = MAX(jcl, 1)
                      jcr = MIN(nx, jcl + 1)
                      fr = xcol - jcl
                      irt = zrow
                      irt = MAX(irt, 1)
                      irb = MIN(nz, irt + 1)
                      fd = zrow - irt
                      tl = tgrid0(irt, jcl) + &
     &                   fd * (tgrid0(irb, jcl) - tgrid0(irt, jcl))
                      tr = tgrid0(irt, jcr) + &
     &                   fd * (tgrid0(irb, jcr) - tgrid0(irt, jcr))
                      t = tl + (tr - tl) * fr
                      IF ((irow == nfault).AND.(nfault > 1)) THEN
                           factor = 0.5
                      ELSE
                           factor = 1.0
                      END IF
                      tgrid1(irow, jcol) = factor * t + &
     &                              (1. - factor) * tgrid0(irow, jcol)
                 END IF
            END DO
       END DO

!   ADD FRICTIONAL HEAT

       deltat = dist * stress * diag / (rhocp * dx * dz)
       DO irow = nz, 2, -1
            jcol = 1 + (nz - irow)
            tgrid1(irow, jcol) = tgrid1(irow, jcol) + deltat
       END DO

!   REPLACE T'S INTO ORIGINAL GRID

       DO jcol = 1, nx
            DO irow = 1, nz
                 tgrid0(irow, jcol) = tgrid1(irow, jcol)
            END DO
       END DO
       RETURN
END SUBROUTINE Advect
